//How it works:
//This add-on interferes with the normal brick button creation function
//to identify all brick datablocks and prevent the normal loading of brick
//buttons, then stores all the datablocks in a variable.

//When the (brick window) load bricks function is finished, the sorting
//function is run. All the datablocks are placed into a GuiTextListCtrl
//in a row like this: "064064160\t1342" with the fields being XYZ and
//datablock number, respectively. The sortNumerical function is called on the
//GuiTextListCtrl which then effectively sorts based on X,Y,Z.

//The sorted list is then used to create new buttons by using the
//brick creation function using its normal behavior.

//This results in every category having the constituent bricks being sorted
//by X, Y and Z sizes.

//avoids unnecessary console output when deactivating an already deactivated package
function deactivatePackageIfActive(%p)
{
	for(%a = 0; %a < getNumActivePackages(); %a++)
		if(%p $= getActivePackage(%a))
		{
			deactivatePackage(%p);
			return;
		}
}

function activatePackageIfNotActive(%p)
{
	for(%a = 0; %a < getNumActivePackages(); %a++)
		if(%p $= getActivePackage(%a))
			return;

	activatePackage(%p);
}


deactivatePackageIfActive(sortedBrickList);

package sortedBrickList
{
	function BSD_LoadBricks()
	{
		//$SBL::sorter = new GuiTextListCtrl();

		$SBL::dbcount = 0;
		%r = Parent::BSD_LoadBricks();
		$SBL::canCreateButtons = 1;
		SBL_sort();
		deleteVariables("$SBL::db*");

		//$SBL::sorter.delete();

		$SBL::canCreateButtons = 0;

		return %r;
	}

	function BSD_CreateBrickButton(%db)
	{
		if($SBL::canCreateButtons)
			return Parent::BSD_CreateBrickButton(%db);
		else
		{
			//could probably add these directly to a sorter
			$SBL::db[$SBL::dbcount] = %db;
			$SBL::dbcount += 1;
			//$SBL::sorter.;
		}
	}
};
activatePackageIfNotActive(sortedBrickList);



//Thanks to Zeblote for a neat trick using GuiTextListCtrl to sort using the engine:
//https://forum.blockland.us/index.php?topic=234486.msg6666280#msg6666280

function SBL_sort()
{
	%sorter = new GuiTextListCtrl();

	for(%i = 0; %i < $SBL::dbcount; %i++)
	{
		%db = $SBL::db[%i];

		//max is 64,64,160
		%x = leadingZeros(%db.brickSizeX, 2);
		%y = leadingZeros(%db.brickSizeY, 2);
		%z = leadingZeros(%db.brickSizeZ, 3);

		%vector = %x @ %y @ %z;

		%sorter.addRow(%i, %vector TAB $SBL::db[%i]);
	}

	%sorter.sortNumerical(0, 1);

	for(%i = 0; %i < %sorter.rowCount(); %i++)
	{
		%r = %sorter.getRowText(%i);
		%lastField = getField(%r, getFieldCount(%r) - 1);
		BSD_CreateBrickButton(%lastField);
	}

	%sorter.delete();
}

function leadingZeros(%str, %total_len)
{
	while(strlen(%str) < %total_len)
		%str = "0"@ %str;

	return %str;
}